@@ -18,6 +18,7 @@
  * License, Version 2.0.
  */
 
+#define PY_SSIZE_T_CLEAN
 #include <Python.h>
 #include <sys/stat.h>
 
@@ -62,11 +63,12 @@ static void free_objects(PyObject **objs, Py_ssize_t n)
 /**
  * Get the entries of a tree, prepending the given path.
  *
- * :param path: The path to prepend, without trailing slashes.
- * :param path_len: The length of path.
- * :param tree: The Tree object to iterate.
- * :param n: Set to the length of result.
- * :return: A (C) array of PyObject pointers to TreeEntry objects for each path
+ * Args:
+ *   path: The path to prepend, without trailing slashes.
+ *   path_len: The length of path.
+ *   tree: The Tree object to iterate.
+ *   n: Set to the length of result.
+ * Returns: A (C) array of PyObject pointers to TreeEntry objects for each path
  *     in tree.
  */
 static PyObject **tree_entries(char *path, Py_ssize_t path_len, PyObject *tree,
@@ -196,8 +198,7 @@ static PyObject *py_merge_entries(PyObject *self, PyObject *args)
 {
 	PyObject *tree1, *tree2, **entries1 = NULL, **entries2 = NULL;
 	PyObject *e1, *e2, *pair, *result = NULL;
-	Py_ssize_t n1 = 0, n2 = 0, i1 = 0, i2 = 0;
-	int path_len;
+	Py_ssize_t n1 = 0, n2 = 0, i1 = 0, i2 = 0, path_len;
 	char *path_str;
 	int cmp;
 
