@@ -100,7 +100,8 @@ DEFAULT_PACK_DELTA_WINDOW_SIZE = 10
 def take_msb_bytes(read, crc32=None):
     """Read bytes marked with most significant bit.
 
-    :param read: Read function
+    Args:
+      read: Read function
     """
     ret = []
     while len(ret) == 0 or ret[-1] & 0x80:
@@ -206,9 +207,10 @@ def read_zlib_chunks(read_some, unpacked, include_comp=False,
     This function requires that the buffer have additional data following the
     compressed data, which is guaranteed to be the case for git pack files.
 
-    :param read_some: Read function that returns at least one byte, but may
+    Args:
+      read_some: Read function that returns at least one byte, but may
         return less than the requested size.
-    :param unpacked: An UnpackedObject to write result data to. If its crc32
+      unpacked: An UnpackedObject to write result data to. If its crc32
         attr is not None, the CRC32 of the compressed bytes will be computed
         using this starting CRC32.
         After this function, will have the following attrs set:
@@ -216,10 +218,11 @@ def read_zlib_chunks(read_some, unpacked, include_comp=False,
         * decomp_chunks
         * decomp_len
         * crc32
-    :param include_comp: If True, include compressed data in the result.
-    :param buffer_size: Size of the read buffer.
-    :return: Leftover unused data from the decompression.
-    :raise zlib.error: if a decompression error occurred.
+      include_comp: If True, include compressed data in the result.
+      buffer_size: Size of the read buffer.
+    Returns: Leftover unused data from the decompression.
+    Raises:
+      zlib.error: if a decompression error occurred.
     """
     if unpacked.decomp_len <= -1:
         raise ValueError('non-negative zlib data stream size expected')
@@ -263,8 +266,9 @@ def read_zlib_chunks(read_some, unpacked, include_comp=False,
 def iter_sha1(iter):
     """Return the hexdigest of the SHA1 over a set of names.
 
-    :param iter: Iterator over string objects
-    :return: 40-byte hex sha1 digest
+    Args:
+      iter: Iterator over string objects
+    Returns: 40-byte hex sha1 digest
     """
     sha = sha1()
     for name in iter:
@@ -275,8 +279,9 @@ def iter_sha1(iter):
 def load_pack_index(path):
     """Load an index file by path.
 
-    :param filename: Path to the index file
-    :return: A PackIndex loaded from the given path
+    Args:
+      filename: Path to the index file
+    Returns: A PackIndex loaded from the given path
     """
     with GitFile(path, 'rb') as f:
         return load_pack_index_file(path, f)
@@ -307,9 +312,10 @@ def _load_file_contents(f, size=None):
 def load_pack_index_file(path, f):
     """Load an index file from a file-like object.
 
-    :param path: Path for the index file
-    :param f: File-like object
-    :return: A PackIndex loaded from the given file
+    Args:
+      path: Path for the index file
+      f: File-like object
+    Returns: A PackIndex loaded from the given file
     """
     contents, size = _load_file_contents(f)
     if contents[:4] == b'\377tOc':
@@ -326,11 +332,12 @@ def load_pack_index_file(path, f):
 def bisect_find_sha(start, end, sha, unpack_name):
     """Find a SHA in a data blob with sorted SHAs.
 
-    :param start: Start index of range to search
-    :param end: End index of range to search
-    :param sha: Sha to find
-    :param unpack_name: Callback to retrieve SHA by index
-    :return: Index of the SHA, or None if it wasn't found
+    Args:
+      start: Start index of range to search
+      end: End index of range to search
+      sha: Sha to find
+      unpack_name: Callback to retrieve SHA by index
+    Returns: Index of the SHA, or None if it wasn't found
     """
     assert start <= end
     while start <= end:
@@ -376,7 +383,7 @@ class PackIndex(object):
     def iterentries(self):
         """Iterate over the entries in this pack index.
 
-        :return: iterator over tuples with object name, offset in packfile and
+        Returns: iterator over tuples with object name, offset in packfile and
             crc32 checksum.
         """
         raise NotImplementedError(self.iterentries)
@@ -384,7 +391,7 @@ class PackIndex(object):
     def get_pack_checksum(self):
         """Return the SHA1 checksum stored for the corresponding packfile.
 
-        :return: 20-byte binary digest
+        Returns: 20-byte binary digest
         """
         raise NotImplementedError(self.get_pack_checksum)
 
@@ -418,14 +425,15 @@ class PackIndex(object):
     def _object_index(self, sha):
         """See object_index.
 
-        :param sha: A *binary* SHA string. (20 characters long)_
+        Args:
+          sha: A *binary* SHA string. (20 characters long)_
         """
         raise NotImplementedError(self._object_index)
 
     def objects_sha1(self):
         """Return the hex SHA1 over all the shas of all objects in this pack.
 
-        :note: This is used for the filename of the pack.
+        Note: This is used for the filename of the pack.
         """
         return iter_sha1(self._itersha())
 
@@ -440,8 +448,9 @@ class MemoryPackIndex(PackIndex):
     def __init__(self, entries, pack_checksum=None):
         """Create a new MemoryPackIndex.
 
-        :param entries: Sequence of name, idx, crc32 (sorted)
-        :param pack_checksum: Optional pack checksum
+        Args:
+          entries: Sequence of name, idx, crc32 (sorted)
+          pack_checksum: Optional pack checksum
         """
         self._by_sha = {}
         self._by_index = {}
@@ -524,7 +533,7 @@ class FilePackIndex(PackIndex):
     def _unpack_entry(self, i):
         """Unpack the i-th entry in the index file.
 
-        :return: Tuple with object name (SHA), offset in pack file and CRC32
+        Returns: Tuple with object name (SHA), offset in pack file and CRC32
             checksum (if known).
         """
         raise NotImplementedError(self._unpack_entry)
@@ -549,7 +558,7 @@ class FilePackIndex(PackIndex):
     def iterentries(self):
         """Iterate over the entries in this pack index.
 
-        :return: iterator over tuples with object name, offset in packfile and
+        Returns: iterator over tuples with object name, offset in packfile and
             crc32 checksum.
         """
         for i in range(len(self)):
@@ -573,28 +582,29 @@ class FilePackIndex(PackIndex):
     def calculate_checksum(self):
         """Calculate the SHA1 checksum over this pack index.
 
-        :return: This is a 20-byte binary digest
+        Returns: This is a 20-byte binary digest
         """
         return sha1(self._contents[:-20]).digest()
 
     def get_pack_checksum(self):
         """Return the SHA1 checksum stored for the corresponding packfile.
 
-        :return: 20-byte binary digest
+        Returns: 20-byte binary digest
         """
         return bytes(self._contents[-40:-20])
 
     def get_stored_checksum(self):
         """Return the SHA1 checksum stored for this index.
 
-        :return: 20-byte binary digest
+        Returns: 20-byte binary digest
         """
         return bytes(self._contents[-20:])
 
     def _object_index(self, sha):
         """See object_index.
 
-        :param sha: A *binary* SHA string. (20 characters long)_
+        Args:
+          sha: A *binary* SHA string. (20 characters long)_
         """
         assert len(sha) == 20
         idx = ord(sha[:1])
@@ -679,8 +689,9 @@ class PackIndex2(FilePackIndex):
 def read_pack_header(read):
     """Read the header of a pack file.
 
-    :param read: Read function
-    :return: Tuple of (pack version, number of objects). If no data is
+    Args:
+      read: Read function
+    Returns: Tuple of (pack version, number of objects). If no data is
         available to read, returns (None, None).
     """
     header = read(12)
@@ -706,15 +717,16 @@ def unpack_object(read_all, read_some=None, compute_crc32=False,
                   include_comp=False, zlib_bufsize=_ZLIB_BUFSIZE):
     """Unpack a Git object.
 
-    :param read_all: Read function that blocks until the number of requested
+    Args:
+      read_all: Read function that blocks until the number of requested
         bytes are read.
-    :param read_some: Read function that returns at least one byte, but may not
+      read_some: Read function that returns at least one byte, but may not
         return the number of bytes requested.
-    :param compute_crc32: If True, compute the CRC32 of the compressed data. If
+      compute_crc32: If True, compute the CRC32 of the compressed data. If
         False, the returned CRC32 will be None.
-    :param include_comp: If True, include compressed data in the result.
-    :param zlib_bufsize: An optional buffer size for zlib operations.
-    :return: A tuple of (unpacked, unused), where unused is the unused data
+      include_comp: If True, include compressed data in the result.
+      zlib_bufsize: An optional buffer size for zlib operations.
+    Returns: A tuple of (unpacked, unused), where unused is the unused data
         leftover from decompression, and unpacked in an UnpackedObject with
         the following attrs set:
 
@@ -799,8 +811,9 @@ class PackStreamReader(object):
         As a side effect, update the verifier's hash (excluding the last 20
         bytes read).
 
-        :param read: The read callback to read from.
-        :param size: The maximum number of bytes to read; the particular
+        Args:
+          read: The read callback to read from.
+          size: The maximum number of bytes to read; the particular
             behavior is callback-specific.
         """
         data = read(size)
@@ -860,9 +873,10 @@ class PackStreamReader(object):
     def read_objects(self, compute_crc32=False):
         """Read the objects in this pack file.
 
-        :param compute_crc32: If True, compute the CRC32 of the compressed
+        Args:
+          compute_crc32: If True, compute the CRC32 of the compressed
             data. If False, the returned CRC32 will be None.
-        :return: Iterator over UnpackedObjects with the following members set:
+        Returns: Iterator over UnpackedObjects with the following members set:
             offset
             obj_type_num
             obj_chunks (for non-delta types)
@@ -870,10 +884,11 @@ class PackStreamReader(object):
             decomp_chunks
             decomp_len
             crc32 (if compute_crc32 is True)
-        :raise ChecksumMismatch: if the checksum of the pack contents does not
+        Raises:
+          ChecksumMismatch: if the checksum of the pack contents does not
             match the checksum in the pack trailer.
-        :raise zlib.error: if an error occurred during zlib decompression.
-        :raise IOError: if an error occurred writing to the output file.
+          zlib.error: if an error occurred during zlib decompression.
+          IOError: if an error occurred writing to the output file.
         """
         pack_version, self._num_objects = read_pack_header(self.read)
         if pack_version is None:
@@ -917,12 +932,13 @@ class PackStreamCopier(PackStreamReader):
     def __init__(self, read_all, read_some, outfile, delta_iter=None):
         """Initialize the copier.
 
-        :param read_all: Read function that blocks until the number of
+        Args:
+          read_all: Read function that blocks until the number of
             requested bytes are read.
-        :param read_some: Read function that returns at least one byte, but may
+          read_some: Read function that returns at least one byte, but may
             not return the number of bytes requested.
-        :param outfile: File-like object to write output through.
-        :param delta_iter: Optional DeltaChainIterator to record deltas as we
+          outfile: File-like object to write output through.
+          delta_iter: Optional DeltaChainIterator to record deltas as we
             read them.
         """
         super(PackStreamCopier, self).__init__(read_all, read_some=read_some)
@@ -964,12 +980,13 @@ def obj_sha(type, chunks):
 def compute_file_sha(f, start_ofs=0, end_ofs=0, buffer_size=1 << 16):
     """Hash a portion of a file into a new SHA.
 
-    :param f: A file-like object to read from that supports seek().
-    :param start_ofs: The offset in the file to start reading at.
-    :param end_ofs: The offset in the file to end reading at, relative to the
+    Args:
+      f: A file-like object to read from that supports seek().
+      start_ofs: The offset in the file to start reading at.
+      end_ofs: The offset in the file to end reading at, relative to the
         end of the file.
-    :param buffer_size: A buffer size for reading.
-    :return: A new SHA object updated with data read from the file.
+      buffer_size: A buffer size for reading.
+    Returns: A new SHA object updated with data read from the file.
     """
     sha = sha1()
     f.seek(0, SEEK_END)
@@ -1078,7 +1095,7 @@ class PackData(object):
     def calculate_checksum(self):
         """Calculate the checksum for this pack.
 
-        :return: 20-byte binary SHA1 digest
+        Returns: 20-byte binary SHA1 digest
         """
         return compute_file_sha(self._file, end_ofs=-20).digest()
 
@@ -1102,7 +1119,7 @@ class PackData(object):
     def resolve_object(self, offset, type, obj, get_ref=None):
         """Resolve an object, possibly resolving deltas when necessary.
 
-        :return: Tuple with object type and contents.
+        Returns: Tuple with object type and contents.
         """
         # Walk down the delta chain, building a stack of deltas to reach
         # the requested object.
@@ -1174,9 +1191,10 @@ class PackData(object):
     def iterentries(self, progress=None):
         """Yield entries summarizing the contents of this pack.
 
-        :param progress: Progress function, called with current and total
+        Args:
+          progress: Progress function, called with current and total
             object count.
-        :return: iterator of tuples with (sha, offset, crc32)
+        Returns: iterator of tuples with (sha, offset, crc32)
         """
         num_objects = self._num_objects
         resolve_ext_ref = (
@@ -1191,9 +1209,10 @@ class PackData(object):
     def sorted_entries(self, progress=None):
         """Return entries in this pack, sorted by SHA.
 
-        :param progress: Progress function, called with current and total
+        Args:
+          progress: Progress function, called with current and total
             object count
-        :return: List of tuples with (sha, offset, crc32)
+        Returns: List of tuples with (sha, offset, crc32)
         """
         ret = sorted(self.iterentries(progress=progress))
         return ret
@@ -1201,9 +1220,10 @@ class PackData(object):
     def create_index_v1(self, filename, progress=None):
         """Create a version 1 file for this data file.
 
-        :param filename: Index filename.
-        :param progress: Progress report function
-        :return: Checksum of index file
+        Args:
+          filename: Index filename.
+          progress: Progress report function
+        Returns: Checksum of index file
         """
         entries = self.sorted_entries(progress=progress)
         with GitFile(filename, 'wb') as f:
@@ -1212,9 +1232,10 @@ class PackData(object):
     def create_index_v2(self, filename, progress=None):
         """Create a version 2 index file for this data file.
 
-        :param filename: Index filename.
-        :param progress: Progress report function
-        :return: Checksum of index file
+        Args:
+          filename: Index filename.
+          progress: Progress report function
+        Returns: Checksum of index file
         """
         entries = self.sorted_entries(progress=progress)
         with GitFile(filename, 'wb') as f:
@@ -1224,9 +1245,10 @@ class PackData(object):
                      version=2):
         """Create an  index file for this data file.
 
-        :param filename: Index filename.
-        :param progress: Progress report function
-        :return: Checksum of index file
+        Args:
+          filename: Index filename.
+          progress: Progress report function
+        Returns: Checksum of index file
         """
         if version == 1:
             return self.create_index_v1(filename, progress)
@@ -1481,10 +1503,11 @@ class SHA1Writer(object):
 def pack_object_header(type_num, delta_base, size):
     """Create a pack object header for the given object info.
 
-    :param type_num: Numeric type of the object.
-    :param delta_base: Delta base offset or ref, or None for whole objects.
-    :param size: Uncompressed object size.
-    :return: A header for a packed object.
+    Args:
+      type_num: Numeric type of the object.
+      delta_base: Delta base offset or ref, or None for whole objects.
+      size: Uncompressed object size.
+    Returns: A header for a packed object.
     """
     header = []
     c = (type_num << 4) | (size & 15)
@@ -1511,10 +1534,11 @@ def pack_object_header(type_num, delta_base, size):
 def write_pack_object(f, type, object, sha=None):
     """Write pack object to a file.
 
-    :param f: File to write to
-    :param type: Numeric type of the object
-    :param object: Object to write
-    :return: Tuple with offset at which the object was written, and crc32
+    Args:
+      f: File to write to
+      type: Numeric type of the object
+      object: Object to write
+    Returns: Tuple with offset at which the object was written, and crc32
     """
     if type in DELTA_TYPES:
         delta_base, object = object
@@ -1534,12 +1558,13 @@ def write_pack_object(f, type, object, sha=None):
 def write_pack(filename, objects, deltify=None, delta_window_size=None):
     """Write a new pack data file.
 
-    :param filename: Path to the new pack file (without .pack extension)
-    :param objects: Iterable of (object, path) tuples to write.
+    Args:
+      filename: Path to the new pack file (without .pack extension)
+      objects: Iterable of (object, path) tuples to write.
         Should provide __len__
-    :param window_size: Delta window size
-    :param deltify: Whether to deltify pack objects
-    :return: Tuple with checksum of pack file and index file
+      window_size: Delta window size
+      deltify: Whether to deltify pack objects
+    Returns: Tuple with checksum of pack file and index file
     """
     with GitFile(filename + '.pack', 'wb') as f:
         entries, data_sum = write_pack_objects(
@@ -1559,9 +1584,10 @@ def write_pack_header(f, num_objects):
 def deltify_pack_objects(objects, window_size=None):
     """Generate deltas for pack objects.
 
-    :param objects: An iterable of (object, path) tuples to deltify.
-    :param window_size: Window size; None for default
-    :return: Iterator over type_num, object id, delta_base, content
+    Args:
+      objects: An iterable of (object, path) tuples to deltify.
+      window_size: Window size; None for default
+    Returns: Iterator over type_num, object id, delta_base, content
         delta_base is None for full text entries
     """
     # TODO(jelmer): Use threads
@@ -1596,8 +1622,9 @@ def deltify_pack_objects(objects, window_size=None):
 def pack_objects_to_data(objects):
     """Create pack data from objects
 
-    :param objects: Pack objects
-    :return: Tuples with (type_num, hexdigest, delta base, object chunks)
+    Args:
+      objects: Pack objects
+    Returns: Tuples with (type_num, hexdigest, delta base, object chunks)
     """
     count = len(objects)
     return (count,
@@ -1608,13 +1635,14 @@ def pack_objects_to_data(objects):
 def write_pack_objects(f, objects, delta_window_size=None, deltify=None):
     """Write a new pack data file.
 
-    :param f: File to write to
-    :param objects: Iterable of (object, path) tuples to write.
+    Args:
+      f: File to write to
+      objects: Iterable of (object, path) tuples to write.
         Should provide __len__
-    :param window_size: Sliding window size for searching for deltas;
+      window_size: Sliding window size for searching for deltas;
                         Set to None for default window size.
-    :param deltify: Whether to deltify objects
-    :return: Dict mapping id -> (offset, crc32 checksum), pack checksum
+      deltify: Whether to deltify objects
+    Returns: Dict mapping id -> (offset, crc32 checksum), pack checksum
     """
     if deltify is None:
         # PERFORMANCE/TODO(jelmer): This should be enabled but is *much* too
@@ -1632,11 +1660,12 @@ def write_pack_objects(f, objects, delta_window_size=None, deltify=None):
 def write_pack_data(f, num_records, records, progress=None):
     """Write a new pack data file.
 
-    :param f: File to write to
-    :param num_records: Number of records
-    :param records: Iterator over type_num, object_id, delta_base, raw
-    :param progress: Function to report progress to
-    :return: Dict mapping id -> (offset, crc32 checksum), pack checksum
+    Args:
+      f: File to write to
+      num_records: Number of records
+      records: Iterator over type_num, object_id, delta_base, raw
+      progress: Function to report progress to
+    Returns: Dict mapping id -> (offset, crc32 checksum), pack checksum
     """
     # Write the pack
     entries = {}
@@ -1665,11 +1694,12 @@ def write_pack_data(f, num_records, records, progress=None):
 def write_pack_index_v1(f, entries, pack_checksum):
     """Write a new pack index file.
 
-    :param f: A file-like object to write to
-    :param entries: List of tuples with object name (sha), offset_in_pack,
+    Args:
+      f: A file-like object to write to
+      entries: List of tuples with object name (sha), offset_in_pack,
         and crc32_checksum.
-    :param pack_checksum: Checksum of the pack file.
-    :return: The SHA of the written index file
+      pack_checksum: Checksum of the pack file.
+    Returns: The SHA of the written index file
     """
     f = SHA1Writer(f)
     fan_out_table = defaultdict(lambda: 0)
@@ -1723,8 +1753,9 @@ def _encode_copy_operation(start, length):
 def create_delta(base_buf, target_buf):
     """Use python difflib to work out how to transform base_buf to target_buf.
 
-    :param base_buf: Base buffer
-    :param target_buf: Target buffer
+    Args:
+      base_buf: Base buffer
+      target_buf: Target buffer
     """
     assert isinstance(base_buf, bytes)
     assert isinstance(target_buf, bytes)
@@ -1766,8 +1797,9 @@ def create_delta(base_buf, target_buf):
 def apply_delta(src_buf, delta):
     """Based on the similar function in git's patch-delta.c.
 
-    :param src_buf: Source buffer
-    :param delta: Delta instructions
+    Args:
+      src_buf: Source buffer
+      delta: Delta instructions
     """
     if not isinstance(src_buf, bytes):
         src_buf = b''.join(src_buf)
@@ -1833,11 +1865,12 @@ def apply_delta(src_buf, delta):
 def write_pack_index_v2(f, entries, pack_checksum):
     """Write a new pack index file.
 
-    :param f: File-like object to write to
-    :param entries: List of tuples with object name (sha), offset_in_pack, and
+    Args:
+      f: File-like object to write to
+      entries: List of tuples with object name (sha), offset_in_pack, and
         crc32_checksum.
-    :param pack_checksum: Checksum of the pack file.
-    :return: The SHA of the index file written
+      pack_checksum: Checksum of the pack file.
+    Returns: The SHA of the index file written
     """
     f = SHA1Writer(f)
     f.write(b'\377tOc')  # Magic!
@@ -1917,7 +1950,7 @@ class Pack(object):
     def index(self):
         """The index being used.
 
-        :note: This may be an in-memory index
+        Note: This may be an in-memory index
         """
         if self._idx is None:
             self._idx = self._idx_load()
@@ -1961,7 +1994,8 @@ class Pack(object):
     def check(self):
         """Check the integrity of this pack.
 
-        :raise ChecksumMismatch: if a checksum for the index or data is wrong
+        Raises:
+          ChecksumMismatch: if a checksum for the index or data is wrong
         """
         self.index.check()
         self.data.check()
@@ -1983,8 +2017,9 @@ class Pack(object):
     def get_raw_unresolved(self, sha1):
         """Get raw unresolved data for a SHA.
 
-        :param sha1: SHA to return data for
-        :return: Tuple with pack object type, delta base (if applicable),
+        Args:
+          sha1: SHA to return data for
+        Returns: Tuple with pack object type, delta base (if applicable),
             list of data chunks
         """
         offset = self.index.object_index(sha1)
@@ -2015,7 +2050,7 @@ class Pack(object):
     def pack_tuples(self):
         """Provide an iterable for use with write_pack_objects.
 
-        :return: Object that can iterate over (object, path) tuples
+        Returns: Object that can iterate over (object, path) tuples
             and provides __len__
         """
         class PackTupleIterable(object):
@@ -2034,9 +2069,10 @@ class Pack(object):
     def keep(self, msg=None):
         """Add a .keep file for the pack, preventing git from garbage collecting it.
 
-        :param msg: A message written inside the .keep file; can be used later
+        Args:
+          msg: A message written inside the .keep file; can be used later
             to determine whether or not a .keep file is obsolete.
-        :return: The path of the .keep file, as a string.
+        Returns: The path of the .keep file, as a string.
         """
         keepfile_name = '%s.keep' % self._basename
         with GitFile(keepfile_name, 'wb') as keepfile:
