@@ -65,6 +65,7 @@ from io import BytesIO, RawIOBase
 import datetime
 import os
 import posixpath
+import shutil
 import stat
 import sys
 import time
@@ -123,6 +124,7 @@ from dulwich.protocol import (
     )
 from dulwich.refs import (
     ANNOTATED_TAG_SUFFIX,
+    LOCAL_BRANCH_PREFIX,
     strip_peeled_refs,
 )
 from dulwich.repo import (BaseRepo, Repo)
@@ -154,12 +156,14 @@ class NoneStream(RawIOBase):
         return None
 
 
-default_bytes_out_stream = getattr(
-        sys.stdout, 'buffer', sys.stdout
-    ) or NoneStream()
-default_bytes_err_stream = getattr(
-        sys.stderr, 'buffer', sys.stderr
-    ) or NoneStream()
+if sys.version_info[0] == 2:
+    default_bytes_out_stream = sys.stdout or NoneStream()
+    default_bytes_err_stream = sys.stderr or NoneStream()
+else:
+    default_bytes_out_stream = (
+        getattr(sys.stdout, 'buffer', None) or NoneStream())
+    default_bytes_err_stream = (
+        getattr(sys.stderr, 'buffer', None) or NoneStream())
 
 
 DEFAULT_ENCODING = 'utf-8'
@@ -196,9 +200,10 @@ def path_to_tree_path(repopath, path):
     """Convert a path to a path usable in an index, e.g. bytes and relative to
     the repository root.
 
-    :param repopath: Repository path, absolute or relative to the cwd
-    :param path: A path, absolute or relative to the cwd
-    :return: A path formatted for use in e.g. an index
+    Args:
+      repopath: Repository path, absolute or relative to the cwd
+      path: A path, absolute or relative to the cwd
+    Returns: A path formatted for use in e.g. an index
     """
     if not isinstance(path, bytes):
         path = path.encode(sys.getfilesystemencoding())
@@ -216,16 +221,17 @@ def archive(repo, committish=None, outstream=default_bytes_out_stream,
             errstream=default_bytes_err_stream):
     """Create an archive.
 
-    :param repo: Path of repository for which to generate an archive.
-    :param committish: Commit SHA1 or ref to use
-    :param outstream: Output stream (defaults to stdout)
-    :param errstream: Error stream (defaults to stderr)
+    Args:
+      repo: Path of repository for which to generate an archive.
+      committish: Commit SHA1 or ref to use
+      outstream: Output stream (defaults to stdout)
+      errstream: Error stream (defaults to stderr)
     """
 
     if committish is None:
         committish = "HEAD"
     with open_repo_closing(repo) as repo_obj:
-        c = repo_obj[committish]
+        c = parse_commit(repo_obj, committish)
         for chunk in tar_stream(
                 repo_obj.object_store, repo_obj.object_store[c.tree],
                 c.commit_time):
@@ -235,7 +241,8 @@ def archive(repo, committish=None, outstream=default_bytes_out_stream,
 def update_server_info(repo="."):
     """Update server info files for a repository.
 
-    :param repo: path to the repository
+    Args:
+      repo: path to the repository
     """
     with open_repo_closing(repo) as r:
         server_update_server_info(r)
@@ -244,9 +251,10 @@ def update_server_info(repo="."):
 def symbolic_ref(repo, ref_name, force=False):
     """Set git symbolic ref into HEAD.
 
-    :param repo: path to the repository
-    :param ref_name: short name of the new ref
-    :param force: force settings without checking if it exists in refs/heads
+    Args:
+      repo: path to the repository
+      ref_name: short name of the new ref
+      force: force settings without checking if it exists in refs/heads
     """
     with open_repo_closing(repo) as repo_obj:
         ref_path = _make_branch_ref(ref_name)
@@ -258,11 +266,12 @@ def symbolic_ref(repo, ref_name, force=False):
 def commit(repo=".", message=None, author=None, committer=None, encoding=None):
     """Create a new commit.
 
-    :param repo: Path to repository
-    :param message: Optional commit message
-    :param author: Optional author name and email
-    :param committer: Optional committer name and email
-    :return: SHA1 of the new commit
+    Args:
+      repo: Path to repository
+      message: Optional commit message
+      author: Optional author name and email
+      committer: Optional committer name and email
+    Returns: SHA1 of the new commit
     """
     # FIXME: Support --all argument
     # FIXME: Support --signoff argument
@@ -281,10 +290,11 @@ def commit(repo=".", message=None, author=None, committer=None, encoding=None):
 def commit_tree(repo, tree, message=None, author=None, committer=None):
     """Create a new commit object.
 
-    :param repo: Path to repository
-    :param tree: An existing tree object
-    :param author: Optional author name and email
-    :param committer: Optional committer name and email
+    Args:
+      repo: Path to repository
+      tree: An existing tree object
+      author: Optional author name and email
+      committer: Optional committer name and email
     """
     with open_repo_closing(repo) as r:
         return r.do_commit(
@@ -294,9 +304,10 @@ def commit_tree(repo, tree, message=None, author=None, committer=None):
 def init(path=".", bare=False):
     """Create a new git repository.
 
-    :param path: Path to repository.
-    :param bare: Whether to create a bare repository.
-    :return: A Repo instance
+    Args:
+      path: Path to repository.
+      bare: Whether to create a bare repository.
+    Returns: A Repo instance
     """
     if not os.path.exists(path):
         os.mkdir(path)
@@ -312,15 +323,16 @@ def clone(source, target=None, bare=False, checkout=None,
           origin=b"origin", depth=None, **kwargs):
     """Clone a local or remote git repository.
 
-    :param source: Path or URL for source repository
-    :param target: Path to target repository (optional)
-    :param bare: Whether or not to create a bare repository
-    :param checkout: Whether or not to check-out HEAD after cloning
-    :param errstream: Optional stream to write progress to
-    :param outstream: Optional stream to write progress to (deprecated)
-    :param origin: Name of remote from the repository used to clone
-    :param depth: Depth to fetch at
-    :return: The new repository
+    Args:
+      source: Path or URL for source repository
+      target: Path to target repository (optional)
+      bare: Whether or not to create a bare repository
+      checkout: Whether or not to check-out HEAD after cloning
+      errstream: Optional stream to write progress to
+      outstream: Optional stream to write progress to (deprecated)
+      origin: Name of remote from the repository used to clone
+      depth: Depth to fetch at
+    Returns: The new repository
     """
     # TODO(jelmer): This code overlaps quite a bit with Repo.clone
     if outstream is not None:
@@ -371,6 +383,7 @@ def clone(source, target=None, bare=False, checkout=None,
             errstream.write(b'Checking out ' + head.id + b'\n')
             r.reset_index(head.tree)
     except BaseException:
+        shutil.rmtree(target)
         r.close()
         raise
 
@@ -380,9 +393,10 @@ def clone(source, target=None, bare=False, checkout=None,
 def add(repo=".", paths=None):
     """Add files to the staging area.
 
-    :param repo: Repository for the files
-    :param paths: Paths to add.  No value passed stages all modified files.
-    :return: Tuple with set of added files and ignored files
+    Args:
+      repo: Repository for the files
+      paths: Paths to add.  No value passed stages all modified files.
+    Returns: Tuple with set of added files and ignored files
     """
     ignored = set()
     with open_repo_closing(repo) as r:
@@ -406,11 +420,64 @@ def add(repo=".", paths=None):
     return (relpaths, ignored)
 
 
+def _is_subdir(subdir, parentdir):
+    """Check whether subdir is parentdir or a subdir of parentdir
+
+        If parentdir or subdir is a relative path, it will be disamgibuated
+        relative to the pwd.
+    """
+    parentdir_abs = os.path.realpath(parentdir) + os.path.sep
+    subdir_abs = os.path.realpath(subdir) + os.path.sep
+    return subdir_abs.startswith(parentdir_abs)
+
+
+# TODO: option to remove ignored files also, in line with `git clean -fdx`
+def clean(repo=".", target_dir=None):
+    """Remove any untracked files from the target directory recursively
+
+    Equivalent to running `git clean -fd` in target_dir.
+
+    Args:
+      repo: Repository where the files may be tracked
+      target_dir: Directory to clean - current directory if None
+    """
+    if target_dir is None:
+        target_dir = os.getcwd()
+
+    with open_repo_closing(repo) as r:
+        if not _is_subdir(target_dir, r.path):
+            raise ValueError("target_dir must be in the repo's working dir")
+
+        index = r.open_index()
+        ignore_manager = IgnoreFilterManager.from_repo(r)
+
+        paths_in_wd = _walk_working_dir_paths(target_dir, r.path)
+        # Reverse file visit order, so that files and subdirectories are
+        # removed before containing directory
+        for ap, is_dir in reversed(list(paths_in_wd)):
+            if is_dir:
+                # All subdirectories and files have been removed if untracked,
+                # so dir contains no tracked files iff it is empty.
+                is_empty = len(os.listdir(ap)) == 0
+                if is_empty:
+                    os.rmdir(ap)
+            else:
+                ip = path_to_tree_path(r.path, ap)
+                is_tracked = ip in index
+
+                rp = os.path.relpath(ap, r.path)
+                is_ignored = ignore_manager.is_ignored(rp)
+
+                if not is_tracked and not is_ignored:
+                    os.remove(ap)
+
+
 def remove(repo=".", paths=None, cached=False):
     """Remove files from the staging area.
 
-    :param repo: Repository for the files
-    :param paths: Paths to remove
+    Args:
+      repo: Repository for the files
+      paths: Paths to remove
     """
     with open_repo_closing(repo) as r:
         index = r.open_index()
@@ -456,16 +523,27 @@ rm = remove
 
 
 def commit_decode(commit, contents, default_encoding=DEFAULT_ENCODING):
-    if commit.encoding is not None:
-        return contents.decode(commit.encoding, "replace")
-    return contents.decode(default_encoding, "replace")
+    if commit.encoding:
+        encoding = commit.encoding.decode('ascii')
+    else:
+        encoding = default_encoding
+    return contents.decode(encoding, "replace")
+
+
+def commit_encode(commit, contents, default_encoding=DEFAULT_ENCODING):
+    if commit.encoding:
+        encoding = commit.encoding.decode('ascii')
+    else:
+        encoding = default_encoding
+    return contents.encode(encoding)
 
 
 def print_commit(commit, decode, outstream=sys.stdout):
     """Write a human-readable commit log entry.
 
-    :param commit: A `Commit` object
-    :param outstream: A stream file to write to
+    Args:
+      commit: A `Commit` object
+      outstream: A stream file to write to
     """
     outstream.write("-" * 50 + "\n")
     outstream.write("commit: " + commit.id.decode('ascii') + "\n")
@@ -489,12 +567,16 @@ def print_commit(commit, decode, outstream=sys.stdout):
 def print_tag(tag, decode, outstream=sys.stdout):
     """Write a human-readable tag.
 
-    :param tag: A `Tag` object
-    :param decode: Function for decoding bytes to unicode string
-    :param outstream: A stream to write to
+    Args:
+      tag: A `Tag` object
+      decode: Function for decoding bytes to unicode string
+      outstream: A stream to write to
     """
     outstream.write("Tagger: " + decode(tag.tagger) + "\n")
-    outstream.write("Date:   " + decode(tag.tag_time) + "\n")
+    time_tuple = time.gmtime(tag.tag_time + tag.tag_timezone)
+    time_str = time.strftime("%a %b %d %Y %H:%M:%S", time_tuple)
+    timezone_str = format_timezone(tag.tag_timezone).decode('ascii')
+    outstream.write("Date:   " + time_str + " " + timezone_str + "\n")
     outstream.write("\n")
     outstream.write(decode(tag.message) + "\n")
     outstream.write("\n")
@@ -503,10 +585,11 @@ def print_tag(tag, decode, outstream=sys.stdout):
 def show_blob(repo, blob, decode, outstream=sys.stdout):
     """Write a blob to a stream.
 
-    :param repo: A `Repo` object
-    :param blob: A `Blob` object
-    :param decode: Function for decoding bytes to unicode string
-    :param outstream: A stream file to write to
+    Args:
+      repo: A `Repo` object
+      blob: A `Blob` object
+      decode: Function for decoding bytes to unicode string
+      outstream: A stream file to write to
     """
     outstream.write(decode(blob.data))
 
@@ -514,10 +597,11 @@ def show_blob(repo, blob, decode, outstream=sys.stdout):
 def show_commit(repo, commit, decode, outstream=sys.stdout):
     """Show a commit to a stream.
 
-    :param repo: A `Repo` object
-    :param commit: A `Commit` object
-    :param decode: Function for decoding bytes to unicode string
-    :param outstream: Stream to write to
+    Args:
+      repo: A `Repo` object
+      commit: A `Commit` object
+      decode: Function for decoding bytes to unicode string
+      outstream: Stream to write to
     """
     print_commit(commit, decode=decode, outstream=outstream)
     if commit.parents:
@@ -530,18 +614,17 @@ def show_commit(repo, commit, decode, outstream=sys.stdout):
         diffstream,
         repo.object_store, base_tree, commit.tree)
     diffstream.seek(0)
-    outstream.write(
-        diffstream.getvalue().decode(
-                commit.encoding or DEFAULT_ENCODING, 'replace'))
+    outstream.write(commit_decode(commit, diffstream.getvalue()))
 
 
 def show_tree(repo, tree, decode, outstream=sys.stdout):
     """Print a tree to a stream.
 
-    :param repo: A `Repo` object
-    :param tree: A `Tree` object
-    :param decode: Function for decoding bytes to unicode string
-    :param outstream: Stream to write to
+    Args:
+      repo: A `Repo` object
+      tree: A `Tree` object
+      decode: Function for decoding bytes to unicode string
+      outstream: Stream to write to
     """
     for n in tree:
         outstream.write(decode(n) + "\n")
@@ -550,13 +633,14 @@ def show_tree(repo, tree, decode, outstream=sys.stdout):
 def show_tag(repo, tag, decode, outstream=sys.stdout):
     """Print a tag to a stream.
 
-    :param repo: A `Repo` object
-    :param tag: A `Tag` object
-    :param decode: Function for decoding bytes to unicode string
-    :param outstream: Stream to write to
+    Args:
+      repo: A `Repo` object
+      tag: A `Tag` object
+      decode: Function for decoding bytes to unicode string
+      outstream: Stream to write to
     """
     print_tag(tag, decode, outstream)
-    show_object(repo, repo[tag.object[1]], outstream)
+    show_object(repo, repo[tag.object[1]], decode, outstream)
 
 
 def show_object(repo, obj, decode, outstream):
@@ -602,12 +686,13 @@ def log(repo=".", paths=None, outstream=sys.stdout, max_entries=None,
         reverse=False, name_status=False):
     """Write commit logs.
 
-    :param repo: Path to repository
-    :param paths: Optional set of specific paths to print entries for
-    :param outstream: Stream to write log output to
-    :param reverse: Reverse order in which entries are printed
-    :param name_status: Print name status
-    :param max_entries: Optional maximum number of entries to display
+    Args:
+      repo: Path to repository
+      paths: Optional set of specific paths to print entries for
+      outstream: Stream to write log output to
+      reverse: Reverse order in which entries are printed
+      name_status: Print name status
+      max_entries: Optional maximum number of entries to display
     """
     with open_repo_closing(repo) as r:
         walker = r.get_walker(
@@ -626,10 +711,11 @@ def show(repo=".", objects=None, outstream=sys.stdout,
          default_encoding=DEFAULT_ENCODING):
     """Print the changes in a commit.
 
-    :param repo: Path to repository
-    :param objects: Objects to show (defaults to [HEAD])
-    :param outstream: Stream to write to
-    :param default_encoding: Default encoding to use if none is set in the
+    Args:
+      repo: Path to repository
+      objects: Objects to show (defaults to [HEAD])
+      outstream: Stream to write to
+      default_encoding: Default encoding to use if none is set in the
         commit
     """
     if objects is None:
@@ -651,10 +737,11 @@ def show(repo=".", objects=None, outstream=sys.stdout,
 def diff_tree(repo, old_tree, new_tree, outstream=sys.stdout):
     """Compares the content and mode of blobs found via two tree objects.
 
-    :param repo: Path to repository
-    :param old_tree: Id of old tree
-    :param new_tree: Id of new tree
-    :param outstream: Stream to write to
+    Args:
+      repo: Path to repository
+      old_tree: Id of old tree
+      new_tree: Id of new tree
+      outstream: Stream to write to
     """
     with open_repo_closing(repo) as r:
         write_tree_diff(outstream, r.object_store, old_tree, new_tree)
@@ -663,9 +750,10 @@ def diff_tree(repo, old_tree, new_tree, outstream=sys.stdout):
 def rev_list(repo, commits, outstream=sys.stdout):
     """Lists commit objects in reverse chronological order.
 
-    :param repo: Path to repository
-    :param commits: Commits over which to iterate
-    :param outstream: Stream to write to
+    Args:
+      repo: Path to repository
+      commits: Commits over which to iterate
+      outstream: Stream to write to
     """
     with open_repo_closing(repo) as r:
         for entry in r.get_walker(include=[r[c].id for c in commits]):
@@ -681,17 +769,20 @@ def tag(*args, **kwargs):
 
 def tag_create(
         repo, tag, author=None, message=None, annotated=False,
-        objectish="HEAD", tag_time=None, tag_timezone=None):
+        objectish="HEAD", tag_time=None, tag_timezone=None,
+        sign=False):
     """Creates a tag in git via dulwich calls:
 
-    :param repo: Path to repository
-    :param tag: tag string
-    :param author: tag author (optional, if annotated is set)
-    :param message: tag message (optional)
-    :param annotated: whether to create an annotated tag
-    :param objectish: object the tag should point at, defaults to HEAD
-    :param tag_time: Optional time for annotated tag
-    :param tag_timezone: Optional timezone for annotated tag
+    Args:
+      repo: Path to repository
+      tag: tag string
+      author: tag author (optional, if annotated is set)
+      message: tag message (optional)
+      annotated: whether to create an annotated tag
+      objectish: object the tag should point at, defaults to HEAD
+      tag_time: Optional time for annotated tag
+      tag_timezone: Optional timezone for annotated tag
+      sign: GPG Sign the tag
     """
 
     with open_repo_closing(repo) as r:
@@ -702,7 +793,7 @@ def tag_create(
             tag_obj = Tag()
             if author is None:
                 # TODO(jelmer): Don't use repo private method.
-                author = r._get_user_identity()
+                author = r._get_user_identity(r.get_config_stack())
             tag_obj.tagger = author
             tag_obj.message = message
             tag_obj.name = tag
@@ -716,6 +807,11 @@ def tag_create(
             elif isinstance(tag_timezone, str):
                 tag_timezone = parse_timezone(tag_timezone)
             tag_obj.tag_timezone = tag_timezone
+            if sign:
+                import gpg
+                with gpg.Context(armor=True) as c:
+                    tag_obj.signature, unused_result = c.sign(
+                        tag_obj.as_raw_string())
             r.object_store.add_object(tag_obj)
             tag_id = tag_obj.id
         else:
@@ -734,8 +830,9 @@ def list_tags(*args, **kwargs):
 def tag_list(repo, outstream=sys.stdout):
     """List all tags.
 
-    :param repo: Path to repository
-    :param outstream: Stream to write tags to
+    Args:
+      repo: Path to repository
+      outstream: Stream to write tags to
     """
     with open_repo_closing(repo) as r:
         tags = sorted(r.refs.as_dict(b"refs/tags"))
@@ -745,8 +842,9 @@ def tag_list(repo, outstream=sys.stdout):
 def tag_delete(repo, name):
     """Remove a tag.
 
-    :param repo: Path to repository
-    :param name: Name of tag to remove
+    Args:
+      repo: Path to repository
+      name: Name of tag to remove
     """
     with open_repo_closing(repo) as r:
         if isinstance(name, bytes):
@@ -762,9 +860,10 @@ def tag_delete(repo, name):
 def reset(repo, mode, treeish="HEAD"):
     """Reset current HEAD to the specified state.
 
-    :param repo: Path to repository
-    :param mode: Mode ("hard", "soft", "mixed")
-    :param treeish: Treeish to reset to
+    Args:
+      repo: Path to repository
+      mode: Mode ("hard", "soft", "mixed")
+      treeish: Treeish to reset to
     """
 
     if mode != "hard":
@@ -780,11 +879,12 @@ def push(repo, remote_location, refspecs,
          errstream=default_bytes_err_stream, **kwargs):
     """Remote push with dulwich via dulwich.client
 
-    :param repo: Path to repository
-    :param remote_location: Location of the remote
-    :param refspecs: Refs to push to remote
-    :param outstream: A stream file to write output
-    :param errstream: A stream file to write errors
+    Args:
+      repo: Path to repository
+      remote_location: Location of the remote
+      refspecs: Refs to push to remote
+      outstream: A stream file to write output
+      errstream: A stream file to write errors
     """
 
     # Open the repo
@@ -827,11 +927,12 @@ def pull(repo, remote_location=None, refspecs=None,
          errstream=default_bytes_err_stream, **kwargs):
     """Pull from remote via dulwich.client
 
-    :param repo: Path to repository
-    :param remote_location: Location of the remote
-    :param refspec: refspecs to fetch
-    :param outstream: A stream file to write to output
-    :param errstream: A stream file to write to errors
+    Args:
+      repo: Path to repository
+      remote_location: Location of the remote
+      refspec: refspecs to fetch
+      outstream: A stream file to write to output
+      errstream: A stream file to write to errors
     """
     # Open the repo
     with open_repo_closing(repo) as r:
@@ -864,10 +965,11 @@ def pull(repo, remote_location=None, refspecs=None,
 def status(repo=".", ignored=False):
     """Returns staged, unstaged, and untracked changes relative to the HEAD.
 
-    :param repo: Path to repository or repository object
-    :param ignored: Whether to include ignored files in `untracked`
-    :return: GitStatus tuple,
-        staged -    list of staged paths (diff index/HEAD)
+    Args:
+      repo: Path to repository or repository object
+      ignored: Whether to include ignored files in `untracked`
+    Returns: GitStatus tuple,
+        staged -  dict with lists of staged paths (diff index/HEAD)
         unstaged -  list of unstaged paths (diff index/working-tree)
         untracked - list of untracked, un-ignored & non-.git paths
     """
@@ -876,7 +978,11 @@ def status(repo=".", ignored=False):
         tracked_changes = get_tree_changes(r)
         # 2. Get status of unstaged
         index = r.open_index()
-        unstaged_changes = list(get_unstaged_changes(index, r.path))
+        normalizer = r.get_blob_normalizer()
+        filter_callback = normalizer.checkin_normalize
+        unstaged_changes = list(
+            get_unstaged_changes(index, r.path, filter_callback)
+        )
         ignore_manager = IgnoreFilterManager.from_repo(r)
         untracked_paths = get_untracked_paths(r.path, r.path, index)
         if ignored:
@@ -888,14 +994,13 @@ def status(repo=".", ignored=False):
         return GitStatus(tracked_changes, unstaged_changes, untracked_changes)
 
 
-def get_untracked_paths(frompath, basepath, index):
-    """Get untracked paths.
+def _walk_working_dir_paths(frompath, basepath):
+    """Get path, is_dir for files in working dir from frompath
 
-    ;param frompath: Path to walk
-    :param basepath: Path to compare to
-    :param index: Index to check against
+    Args:
+      frompath: Path to begin walk
+      basepath: Path to compare to
     """
-    # If nothing is specified, add all non-ignored files.
     for dirpath, dirnames, filenames in os.walk(frompath):
         # Skip .git and below.
         if '.git' in dirnames:
@@ -906,8 +1011,25 @@ def get_untracked_paths(frompath, basepath, index):
             filenames.remove('.git')
             if dirpath != basepath:
                 continue
+
+        if dirpath != frompath:
+            yield dirpath, True
+
         for filename in filenames:
-            ap = os.path.join(dirpath, filename)
+            filepath = os.path.join(dirpath, filename)
+            yield filepath, False
+
+
+def get_untracked_paths(frompath, basepath, index):
+    """Get untracked paths.
+
+    Args:
+    ;param frompath: Path to walk
+      basepath: Path to compare to
+      index: Index to check against
+    """
+    for ap, is_dir in _walk_working_dir_paths(frompath, basepath):
+        if not is_dir:
             ip = path_to_tree_path(basepath, ap)
             if ip not in index:
                 yield os.path.relpath(ap, frompath)
@@ -916,8 +1038,9 @@ def get_untracked_paths(frompath, basepath, index):
 def get_tree_changes(repo):
     """Return add/delete/modify changes to tree by comparing index to HEAD.
 
-    :param repo: repo path or object
-    :return: dict with lists for each type of change
+    Args:
+      repo: repo path or object
+    Returns: dict with lists for each type of change
     """
     with open_repo_closing(repo) as r:
         index = r.open_index()
@@ -950,9 +1073,10 @@ def get_tree_changes(repo):
 def daemon(path=".", address=None, port=None):
     """Run a daemon serving Git requests over TCP/IP.
 
-    :param path: Path to the directory to serve.
-    :param address: Optional address to listen on (defaults to ::)
-    :param port: Optional port to listen on (defaults to TCP_GIT_PORT)
+    Args:
+      path: Path to the directory to serve.
+      address: Optional address to listen on (defaults to ::)
+      port: Optional port to listen on (defaults to TCP_GIT_PORT)
     """
     # TODO(jelmer): Support git-daemon-export-ok and --export-all.
     backend = FileSystemBackend(path)
@@ -963,9 +1087,10 @@ def daemon(path=".", address=None, port=None):
 def web_daemon(path=".", address=None, port=None):
     """Run a daemon serving Git requests over HTTP.
 
-    :param path: Path to the directory to serve
-    :param address: Optional address to listen on (defaults to ::)
-    :param port: Optional port to listen on (defaults to 80)
+    Args:
+      path: Path to the directory to serve
+      address: Optional address to listen on (defaults to ::)
+      port: Optional port to listen on (defaults to 80)
     """
     from dulwich.web import (
         make_wsgi_chain,
@@ -984,9 +1109,10 @@ def web_daemon(path=".", address=None, port=None):
 def upload_pack(path=".", inf=None, outf=None):
     """Upload a pack file after negotiating its contents using smart protocol.
 
-    :param path: Path to the repository
-    :param inf: Input stream to communicate with client
-    :param outf: Output stream to communicate with client
+    Args:
+      path: Path to the repository
+      inf: Input stream to communicate with client
+      outf: Output stream to communicate with client
     """
     if outf is None:
         outf = getattr(sys.stdout, 'buffer', sys.stdout)
@@ -1008,9 +1134,10 @@ def upload_pack(path=".", inf=None, outf=None):
 def receive_pack(path=".", inf=None, outf=None):
     """Receive a pack file after negotiating its contents using smart protocol.
 
-    :param path: Path to the repository
-    :param inf: Input stream to communicate with client
-    :param outf: Output stream to communicate with client
+    Args:
+      path: Path to the repository
+      inf: Input stream to communicate with client
+      outf: Output stream to communicate with client
     """
     if outf is None:
         outf = getattr(sys.stdout, 'buffer', sys.stdout)
@@ -1032,7 +1159,7 @@ def receive_pack(path=".", inf=None, outf=None):
 def _make_branch_ref(name):
     if getattr(name, 'encode', None):
         name = name.encode(DEFAULT_ENCODING)
-    return b"refs/heads/" + name
+    return LOCAL_BRANCH_PREFIX + name
 
 
 def _make_tag_ref(name):
@@ -1044,8 +1171,9 @@ def _make_tag_ref(name):
 def branch_delete(repo, name):
     """Delete a branch.
 
-    :param repo: Path to the repository
-    :param name: Name of the branch
+    Args:
+      repo: Path to the repository
+      name: Name of the branch
     """
     with open_repo_closing(repo) as r:
         if isinstance(name, list):
@@ -1059,10 +1187,11 @@ def branch_delete(repo, name):
 def branch_create(repo, name, objectish=None, force=False):
     """Create a branch.
 
-    :param repo: Path to the repository
-    :param name: Name of the new branch
-    :param objectish: Target object to point new branch at (defaults to HEAD)
-    :param force: Force creation of branch, even if it already exists
+    Args:
+      repo: Path to the repository
+      name: Name of the new branch
+      objectish: Target object to point new branch at (defaults to HEAD)
+      force: Force creation of branch, even if it already exists
     """
     with open_repo_closing(repo) as r:
         if objectish is None:
@@ -1080,25 +1209,48 @@ def branch_create(repo, name, objectish=None, force=False):
 def branch_list(repo):
     """List all branches.
 
-    :param repo: Path to the repository
+    Args:
+      repo: Path to the repository
     """
     with open_repo_closing(repo) as r:
-        return r.refs.keys(base=b"refs/heads/")
+        return r.refs.keys(base=LOCAL_BRANCH_PREFIX)
+
+
+def active_branch(repo):
+    """Return the active branch in the repository, if any.
+
+    Args:
+      repo: Repository to open
+    Returns:
+      branch name
+    Raises:
+      KeyError: if the repository does not have a working tree
+      IndexError: if HEAD is floating
+    """
+    with open_repo_closing(repo) as r:
+        active_ref = r.refs.follow(b'HEAD')[0][1]
+        if not active_ref.startswith(LOCAL_BRANCH_PREFIX):
+            raise ValueError(active_ref)
+        return active_ref[len(LOCAL_BRANCH_PREFIX):]
 
 
 def fetch(repo, remote_location, remote_name=b'origin', outstream=sys.stdout,
           errstream=default_bytes_err_stream, message=None, depth=None,
-          **kwargs):
+          prune=False, prune_tags=False, **kwargs):
     """Fetch objects from a remote server.
 
-    :param repo: Path to the repository
-    :param remote_location: String identifying a remote server
-    :param remote_name: Name for remote server
-    :param outstream: Output stream (defaults to stdout)
-    :param errstream: Error stream (defaults to stderr)
-    :param message: Reflog message (defaults to b"fetch: from <remote_name>")
-    :param depth: Depth to fetch at
-    :return: Dictionary with refs on the remote
+    Args:
+      repo: Path to the repository
+      remote_location: String identifying a remote server
+      remote_name: Name for remote server
+      outstream: Output stream (defaults to stdout)
+      errstream: Error stream (defaults to stderr)
+      message: Reflog message (defaults to b"fetch: from <remote_name>")
+      depth: Depth to fetch at
+      prune: Prune remote removed refs
+      prune_tags: Prune reomte removed tags
+    Returns:
+      Dictionary with refs on the remote
     """
     if message is None:
         message = b'fetch: from ' + remote_location.encode("utf-8")
@@ -1109,24 +1261,29 @@ def fetch(repo, remote_location, remote_name=b'origin', outstream=sys.stdout,
                                     depth=depth)
         stripped_refs = strip_peeled_refs(fetch_result.refs)
         branches = {
-            n[len(b'refs/heads/'):]: v for (n, v) in stripped_refs.items()
-            if n.startswith(b'refs/heads/')}
+            n[len(LOCAL_BRANCH_PREFIX):]: v for (n, v) in stripped_refs.items()
+            if n.startswith(LOCAL_BRANCH_PREFIX)}
         r.refs.import_refs(
-            b'refs/remotes/' + remote_name, branches, message=message)
+            b'refs/remotes/' + remote_name, branches, message=message,
+            prune=prune)
         tags = {
             n[len(b'refs/tags/'):]: v for (n, v) in stripped_refs.items()
             if n.startswith(b'refs/tags/') and
             not n.endswith(ANNOTATED_TAG_SUFFIX)}
-        r.refs.import_refs(b'refs/tags', tags, message=message)
+        r.refs.import_refs(
+            b'refs/tags', tags, message=message,
+            prune=prune_tags)
     return fetch_result.refs
 
 
 def ls_remote(remote, config=None, **kwargs):
     """List the refs in a remote.
 
-    :param remote: Remote repository location
-    :param config: Configuration to use
-    :return: Dictionary with remote refs
+    Args:
+      remote: Remote repository location
+      config: Configuration to use
+    Returns:
+      Dictionary with remote refs
     """
     if config is None:
         config = StackedConfig.default()
@@ -1139,7 +1296,8 @@ def repack(repo):
 
     Currently this only packs loose objects.
 
-    :param repo: Path to the repository
+    Args:
+      repo: Path to the repository
     """
     with open_repo_closing(repo) as r:
         r.object_store.pack_loose_objects()
@@ -1148,10 +1306,11 @@ def repack(repo):
 def pack_objects(repo, object_ids, packf, idxf, delta_window_size=None):
     """Pack objects into a file.
 
-    :param repo: Path to the repository
-    :param object_ids: List of object ids to write
-    :param packf: File-like object to write to
-    :param idxf: File-like object to write to (can be None)
+    Args:
+      repo: Path to the repository
+      object_ids: List of object ids to write
+      packf: File-like object to write to
+      idxf: File-like object to write to (can be None)
     """
     with open_repo_closing(repo) as r:
         entries, data_sum = write_pack_objects(
@@ -1167,11 +1326,12 @@ def ls_tree(repo, treeish=b"HEAD", outstream=sys.stdout, recursive=False,
             name_only=False):
     """List contents of a tree.
 
-    :param repo: Path to the repository
-    :param tree_ish: Tree id to list
-    :param outstream: Output stream (defaults to stdout)
-    :param recursive: Whether to recursively list files
-    :param name_only: Only print item name
+    Args:
+      repo: Path to the repository
+      tree_ish: Tree id to list
+      outstream: Output stream (defaults to stdout)
+      recursive: Whether to recursively list files
+      name_only: Only print item name
     """
     def list_tree(store, treeid, base):
         for (name, mode, sha) in store[treeid].iteritems():
@@ -1191,9 +1351,10 @@ def ls_tree(repo, treeish=b"HEAD", outstream=sys.stdout, recursive=False,
 def remote_add(repo, name, url):
     """Add a remote.
 
-    :param repo: Path to the repository
-    :param name: Remote name
-    :param url: Remote URL
+    Args:
+      repo: Path to the repository
+      name: Remote name
+      url: Remote URL
     """
     if not isinstance(name, bytes):
         name = name.encode(DEFAULT_ENCODING)
@@ -1211,10 +1372,11 @@ def remote_add(repo, name, url):
 def check_ignore(repo, paths, no_index=False):
     """Debug gitignore files.
 
-    :param repo: Path to the repository
-    :param paths: List of paths to check for
-    :param no_index: Don't check index
-    :return: List of ignored files
+    Args:
+      repo: Path to the repository
+      paths: List of paths to check for
+      no_index: Don't check index
+    Returns: List of ignored files
     """
     with open_repo_closing(repo) as r:
         index = r.open_index()
@@ -1233,10 +1395,11 @@ def update_head(repo, target, detached=False, new_branch=None):
 
     Note that this does not actually update the working tree.
 
-    :param repo: Path to the repository
-    :param detach: Create a detached head
-    :param target: Branch or committish to switch to
-    :param new_branch: New branch to create
+    Args:
+      repo: Path to the repository
+      detach: Create a detached head
+      target: Branch or committish to switch to
+      new_branch: New branch to create
     """
     with open_repo_closing(repo) as r:
         if new_branch is not None:
@@ -1258,9 +1421,10 @@ def update_head(repo, target, detached=False, new_branch=None):
 def check_mailmap(repo, contact):
     """Check canonical name and email of contact.
 
-    :param repo: Path to the repository
-    :param contact: Contact name and/or email
-    :return: Canonical contact data
+    Args:
+      repo: Path to the repository
+      contact: Contact name and/or email
+    Returns: Canonical contact data
     """
     with open_repo_closing(repo) as r:
         from dulwich.mailmap import Mailmap
@@ -1277,8 +1441,9 @@ def check_mailmap(repo, contact):
 def fsck(repo):
     """Check a repository.
 
-    :param repo: A path to the repository
-    :return: Iterator over errors/warnings
+    Args:
+      repo: A path to the repository
+    Returns: Iterator over errors/warnings
     """
     with open_repo_closing(repo) as r:
         # TODO(jelmer): check pack files
@@ -1325,8 +1490,9 @@ def ls_files(repo):
 def describe(repo):
     """Describe the repository version.
 
-    :param projdir: git repository root
-    :returns: a string description of the current git revision
+    Args:
+      projdir: git repository root
+    Returns: a string description of the current git revision
 
     Examples: "gabcdefh", "v0.1" or "v0.1-5-gabcdefh".
     """
@@ -1394,20 +1560,32 @@ def describe(repo):
 def get_object_by_path(repo, path, committish=None):
     """Get an object by path.
 
-    :param repo: A path to the repository
-    :param path: Path to look up
-    :param committish: Commit to look up path in
-    :return: A `ShaFile` object
+    Args:
+      repo: A path to the repository
+      path: Path to look up
+      committish: Commit to look up path in
+    Returns: A `ShaFile` object
     """
     if committish is None:
         committish = "HEAD"
     # Get the repository
     with open_repo_closing(repo) as r:
-        commit = parse_commit(repo, committish)
+        commit = parse_commit(r, committish)
         base_tree = commit.tree
         if not isinstance(path, bytes):
-            path = path.encode(commit.encoding or DEFAULT_ENCODING)
+            path = commit_encode(commit, path)
         (mode, sha) = tree_lookup_path(
             r.object_store.__getitem__,
             base_tree, path)
         return r[sha]
+
+
+def write_tree(repo):
+    """Write a tree object from the index.
+
+    Args:
+      repo: Repository for which to write tree
+    Returns: tree id for the tree that was written
+    """
+    with open_repo_closing(repo) as r:
+        return r.open_index().commit(r.object_store)
