@@ -185,6 +185,11 @@ class RefsContainerTests(object):
         self.assertEqual([b'refs-0.1', b'refs-0.2'],
                          sorted(self._refs.keys(b'refs/tags')))
 
+    def test_iter(self):
+        actual_keys = set(self._refs.keys())
+        self.assertEqual(set(self._refs), actual_keys)
+        self.assertEqual(set(_TEST_REFS.keys()), actual_keys)
+
     def test_as_dict(self):
         # refs/heads/loop does not show up even if it exists
         expected_refs = dict(_TEST_REFS)
@@ -219,6 +224,10 @@ class RefsContainerTests(object):
             b'HEAD', b'42d06bd4b77fed026b154d16493e5deab78f02ec', nines))
         self.assertEqual(nines, self._refs[b'HEAD'])
 
+        # Setting the ref again is a no-op, but will return True.
+        self.assertTrue(self._refs.set_if_equals(b'HEAD', nines, nines))
+        self.assertEqual(nines, self._refs[b'HEAD'])
+
         self.assertTrue(self._refs.set_if_equals(b'refs/heads/master', None,
                                                  nines))
         self.assertEqual(nines, self._refs[b'refs/heads/master'])
@@ -288,6 +297,32 @@ class RefsContainerTests(object):
             b'refs/tags/refs-0.2', ZERO_SHA))
         self.assertFalse(b'refs/tags/refs-0.2' in self._refs)
 
+    def test_import_refs_name(self):
+        self._refs[b'refs/remotes/origin/other'] = (
+            b'48d01bd4b77fed026b154d16493e5deab78f02ec')
+        self._refs.import_refs(
+            b'refs/remotes/origin',
+            {b'master': b'42d06bd4b77fed026b154d16493e5deab78f02ec'})
+        self.assertEqual(
+            b'42d06bd4b77fed026b154d16493e5deab78f02ec',
+            self._refs[b'refs/remotes/origin/master'])
+        self.assertEqual(
+            b'48d01bd4b77fed026b154d16493e5deab78f02ec',
+            self._refs[b'refs/remotes/origin/other'])
+
+    def test_import_refs_name_prune(self):
+        self._refs[b'refs/remotes/origin/other'] = (
+            b'48d01bd4b77fed026b154d16493e5deab78f02ec')
+        self._refs.import_refs(
+            b'refs/remotes/origin',
+            {b'master': b'42d06bd4b77fed026b154d16493e5deab78f02ec'},
+            prune=True)
+        self.assertEqual(
+            b'42d06bd4b77fed026b154d16493e5deab78f02ec',
+            self._refs[b'refs/remotes/origin/master'])
+        self.assertNotIn(
+            b'refs/remotes/origin/other', self._refs)
+
 
 class DictRefsContainerTests(RefsContainerTests, TestCase):
 
@@ -335,10 +370,37 @@ class DiskRefsContainerTests(RefsContainerTests, TestCase):
 
     def test_setitem(self):
         RefsContainerTests.test_setitem(self)
-        f = open(os.path.join(self._refs.path, b'refs', b'some', b'ref'), 'rb')
-        self.assertEqual(b'42d06bd4b77fed026b154d16493e5deab78f02ec',
-                         f.read()[:40])
-        f.close()
+        path = os.path.join(self._refs.path, b'refs', b'some', b'ref')
+        with open(path, 'rb') as f:
+            self.assertEqual(b'42d06bd4b77fed026b154d16493e5deab78f02ec',
+                             f.read()[:40])
+
+        self.assertRaises(
+            OSError, self._refs.__setitem__,
+            b'refs/some/ref/sub', b'42d06bd4b77fed026b154d16493e5deab78f02ec')
+
+    def test_setitem_packed(self):
+        with open(os.path.join(self._refs.path, b'packed-refs'), 'w') as f:
+            f.write('# pack-refs with: peeled fully-peeled sorted \n')
+            f.write(
+                '42d06bd4b77fed026b154d16493e5deab78f02ec refs/heads/packed\n')
+
+        # It's allowed to set a new ref on a packed ref, the new ref will be
+        # placed outside on refs/
+        self._refs[b'refs/heads/packed'] = (
+            b'3ec9c43c84ff242e3ef4a9fc5bc111fd780a76a8'
+        )
+        packed_ref_path = os.path.join(
+            self._refs.path, b'refs', b'heads', b'packed')
+        with open(packed_ref_path, 'rb') as f:
+            self.assertEqual(
+                b'3ec9c43c84ff242e3ef4a9fc5bc111fd780a76a8',
+                f.read()[:40])
+
+        self.assertRaises(
+            OSError, self._refs.__setitem__,
+            b'refs/heads/packed/sub',
+            b'42d06bd4b77fed026b154d16493e5deab78f02ec')
 
     def test_setitem_symbolic(self):
         ones = b'1' * 40
@@ -486,6 +548,13 @@ class DiskRefsContainerTests(RefsContainerTests, TestCase):
                          self._refs.read_ref(b'refs/heads/packed'))
         self.assertEqual(None, self._refs.read_ref(b'nonexistant'))
 
+    def test_read_loose_ref(self):
+        self._refs[b'refs/heads/foo'] = (
+            b'df6800012397fb85c56e7418dd4eb9405dee075c'
+        )
+
+        self.assertEqual(None, self._refs.read_ref(b'refs/heads/foo/bar'))
+
     def test_non_ascii(self):
         try:
             encoded_ref = u'refs/tags/schön'.encode(
